@@ -1,6 +1,7 @@
 """The IPython kernel implementation"""
 
 import asyncio
+import builtins
 from contextlib import contextmanager
 from functools import partial
 import getpass
@@ -8,10 +9,10 @@ import signal
 import sys
 
 from IPython.core import release
-from ipython_genutils.py3compat import builtin_mod, PY3, unicode_type, safe_unicode
+from ipython_genutils.py3compat import safe_unicode
 from IPython.utils.tokenutil import token_at_cursor, line_at_cursor
 from tornado import gen
-from traitlets import Instance, Type, Any, List, Bool
+from traitlets import Instance, Type, Any, List, Bool, observe, observe_compat
 
 from .comm import CommManager
 from .kernelbase import Kernel as KernelBase
@@ -42,14 +43,18 @@ class IPythonKernel(KernelBase):
     ).tag(config=True)
 
     user_module = Any()
-    def _user_module_changed(self, name, old, new):
+    @observe('user_module')
+    @observe_compat
+    def _user_module_changed(self, change):
         if self.shell is not None:
-            self.shell.user_module = new
+            self.shell.user_module = change['new']
 
     user_ns = Instance(dict, args=None, allow_none=True)
-    def _user_ns_changed(self, name, old, new):
+    @observe('user_ns')
+    @observe_compat
+    def _user_ns_changed(self, change):
         if self.shell is not None:
-            self.shell.user_ns = new
+            self.shell.user_ns = change['new']
             self.shell.init_user_ns()
 
     # A reference to the Python builtin 'raw_input' function.
@@ -127,7 +132,7 @@ class IPythonKernel(KernelBase):
             'name': 'ipython',
             'version': sys.version_info[0]
         },
-        'pygments_lexer': 'ipython%d' % (3 if PY3 else 2),
+        'pygments_lexer': 'ipython%d' % 3,
         'nbconvert_exporter': 'python',
         'file_extension': '.py'
     }
@@ -181,24 +186,15 @@ class IPythonKernel(KernelBase):
         """
         self._allow_stdin = allow_stdin
 
-        if PY3:
-            self._sys_raw_input = builtin_mod.input
-            builtin_mod.input = self.raw_input
-        else:
-            self._sys_raw_input = builtin_mod.raw_input
-            self._sys_eval_input = builtin_mod.input
-            builtin_mod.raw_input = self.raw_input
-            builtin_mod.input = lambda prompt='': eval(self.raw_input(prompt))
+        self._sys_raw_input = builtins.input
+        builtins.input = self.raw_input
+
         self._save_getpass = getpass.getpass
         getpass.getpass = self.getpass
 
     def _restore_input(self):
         """Restore raw_input, getpass"""
-        if PY3:
-            builtin_mod.input = self._sys_raw_input
-        else:
-            builtin_mod.raw_input = self._sys_raw_input
-            builtin_mod.input = self._sys_eval_input
+        builtins.input = self._sys_raw_input
 
         getpass.getpass = self._save_getpass
 
@@ -313,14 +309,14 @@ class IPythonKernel(KernelBase):
             err = res.error_in_exec
 
         if res.success:
-            reply_content[u'status'] = u'ok'
+            reply_content['status'] = 'ok'
         else:
-            reply_content[u'status'] = u'error'
+            reply_content['status'] = 'error'
 
             reply_content.update({
-                u'traceback': shell._last_traceback or [],
-                u'ename': unicode_type(type(err).__name__),
-                u'evalue': safe_unicode(err),
+                'traceback': shell._last_traceback or [],
+                'ename': str(type(err).__name__),
+                'evalue': safe_unicode(err),
             })
 
             # FIXME: deprecated piece for ipyparallel (remove in 5.0):
@@ -339,16 +335,16 @@ class IPythonKernel(KernelBase):
         # At this point, we can tell whether the main code execution succeeded
         # or not.  If it did, we proceed to evaluate user_expressions
         if reply_content['status'] == 'ok':
-            reply_content[u'user_expressions'] = \
+            reply_content['user_expressions'] = \
                          shell.user_expressions(user_expressions or {})
         else:
             # If there was an error, don't even try to compute expressions
-            reply_content[u'user_expressions'] = {}
+            reply_content['user_expressions'] = {}
 
         # Payloads should be retrieved regardless of outcome, so we can both
         # recover partial output (that could have been generated early in a
         # block, before an error) and always clear the payload system.
-        reply_content[u'payload'] = shell.payload_manager.read_payload()
+        reply_content['payload'] = shell.payload_manager.read_payload()
         # Be aggressive about clearing the payload because we don't want
         # it to sit in memory until the next execute_request comes in.
         shell.payload_manager.clear_payload()
@@ -504,16 +500,16 @@ class IPythonKernel(KernelBase):
             # invoke IPython traceback formatting
             shell.showtraceback()
             reply_content = {
-                u'traceback': shell._last_traceback or [],
-                u'ename': unicode_type(type(e).__name__),
-                u'evalue': safe_unicode(e),
+                'traceback': shell._last_traceback or [],
+                'ename': str(type(e).__name__),
+                'evalue': safe_unicode(e),
             }
             # FIXME: deprecated piece for ipyparallel (remove in 5.0):
             e_info = dict(engine_uuid=self.ident, engine_id=self.int_id, method='apply')
             reply_content['engine_info'] = e_info
 
-            self.send_response(self.iopub_socket, u'error', reply_content,
-                                ident=self._topic('error'))
+            self.send_response(self.iopub_socket, 'error', reply_content,
+                               ident=self._topic('error'))
             self.log.info("Exception in apply request:\n%s", '\n'.join(reply_content['traceback']))
             result_buf = []
             reply_content['status'] = 'error'
